package info.frostfs.sdk.pool;

import java.util.ArrayList;
import java.util.Random;

class Sampler {
    private final Object lock = new Object();
    private final Random random = new Random();
    private final double[] probabilities;
    private final int[] alias;

    Sampler(double[] probabilities) {
        ArrayList<Integer> small = new ArrayList<>();
        ArrayList<Integer> large = new ArrayList<>();

        int n = probabilities.length;

        this.probabilities = new double[n];
        this.alias = new int[n];

        // Compute scaled probabilities.
        double[] p = new double[n];

        for (int i = 0; i < n; i++) {
            p[i] = probabilities[i] * n;
            if (p[i] < 1) {
                small.add(i);
            } else {
                large.add(i);
            }
        }

        while (!small.isEmpty() && !large.isEmpty()) {
            int l = small.remove(small.size() - 1);
            int g = large.remove(large.size() - 1);

            this.probabilities[l] = p[l];
            this.alias[l] = g;

            p[g] = p[g] + p[l] - 1;

            if (p[g] < 1) {
                small.add(g);
            } else {
                large.add(g);
            }
        }

        while (!large.isEmpty()) {
            int g = large.remove(large.size() - 1);
            this.probabilities[g] = 1;
        }

        while (!small.isEmpty()) {
            int l = small.remove(small.size() - 1);
            probabilities[l] = 1;
        }
    }

    int next() {
        int n = alias.length;

        int i;
        double f;
        synchronized (lock) {
            i = random.nextInt(n);
            f = random.nextDouble();
        }

        if (f < probabilities[i]) {
            return i;
        }

        return alias[i];
    }
}

