package info.frostfs.sdk.services.impl;

import info.frostfs.sdk.dto.session.SessionToken;
import info.frostfs.sdk.exceptions.FrostFSException;
import info.frostfs.sdk.jdo.ClientEnvironment;
import info.frostfs.sdk.jdo.parameters.CallContext;
import info.frostfs.sdk.jdo.parameters.session.PrmSessionCreate;
import info.frostfs.sdk.services.ContextAccessor;
import info.frostfs.sdk.services.SessionTools;

import static info.frostfs.sdk.constants.ErrorConst.SESSION_CREATE_FAILED;
import static java.util.Objects.isNull;

public class SessionToolsImpl extends ContextAccessor implements SessionTools {

    public SessionToolsImpl(ClientEnvironment clientEnvironment) {
        super(clientEnvironment);
    }

    @Override
    public SessionToken getOrCreateSession(ClientEnvironment env, CallContext ctx) {
        var token = env.getSessionCache().tryGetValue(env.getSessionKey());

        if (isNull(token)) {
            token = env.getFrostFSClient().createSession(new PrmSessionCreate(-1), ctx);
            if (isNull(token)) {
                throw new FrostFSException(SESSION_CREATE_FAILED);
            }

            env.getSessionCache().setValue(env.getSessionKey(), token);
        }

        return token;
    }
}
