package info.frostfs.sdk.services.impl.interceptor;

import io.grpc.*;

import java.time.Clock;


public class MonitoringClientInterceptor implements ClientInterceptor {
    private final Clock clock;
    private final Configuration configuration;
    private final ClientMetrics.Factory clientMetricsFactory;

    private MonitoringClientInterceptor(
            Clock clock, Configuration configuration, ClientMetrics.Factory clientMetricsFactory) {
        this.clock = clock;
        this.configuration = configuration;
        this.clientMetricsFactory = clientMetricsFactory;
    }

    public static MonitoringClientInterceptor create(Configuration configuration) {
        return new MonitoringClientInterceptor(
                Clock.systemDefaultZone(), configuration, new ClientMetrics.Factory(configuration));
    }

    @Override
    public <R, S> ClientCall<R, S> interceptCall(
            MethodDescriptor<R, S> methodDescriptor, CallOptions callOptions, Channel channel) {
        GrpcMethod grpcMethod = GrpcMethod.of(methodDescriptor, channel);
        ClientMetrics metrics = clientMetricsFactory.createMetricsForMethod(grpcMethod);
        return new MonitoringClientCall<>(
                channel.newCall(methodDescriptor, callOptions), metrics, grpcMethod, configuration, clock);
    }
}
