package info.frostfs.sdk.services.impl.rwhelper;

import com.google.protobuf.ByteString;
import frostfs.object.Service;
import info.frostfs.sdk.dto.object.ObjectId;
import info.frostfs.sdk.jdo.ClientEnvironment;
import info.frostfs.sdk.jdo.parameters.object.PrmObjectPutBase;
import info.frostfs.sdk.tools.RequestConstructor;
import info.frostfs.sdk.tools.Verifier;
import lombok.AllArgsConstructor;
import lombok.Getter;

import static info.frostfs.sdk.tools.RequestSigner.sign;

//todo specify a deadline for each stream request, not for the entire stream
@Getter
@AllArgsConstructor
public class ObjectWriter {
    private final ClientEnvironment environment;
    private final PrmObjectPutBase args;
    private final ObjectStreamer streamer;

    public void write(byte[] buffer) {
        var body = Service.PutRequest.Body.newBuilder()
                .setChunk(ByteString.copyFrom(buffer))
                .build();
        var chunkRequest = Service.PutRequest.newBuilder()
                .setBody(body)
                .clearVerifyHeader();

        RequestConstructor.addMetaHeader(chunkRequest, args.getXHeaders());
        sign(chunkRequest, environment.getKey());

        streamer.write(chunkRequest.build());
    }

    public ObjectId complete() {
        var response = streamer.complete();
        Verifier.checkResponse(response);

        return new ObjectId(response.getBody().getObjectId().getValue().toByteArray());
    }
}
