package info.frostfs.sdk.utils;

import com.google.protobuf.Message;
import com.google.protobuf.MessageOrBuilder;
import info.frostfs.sdk.exceptions.ValidationFrostFSException;
import org.apache.commons.lang3.StringUtils;

import static info.frostfs.sdk.constants.ErrorConst.SOME_PARAM_IS_MISSING;
import static java.util.Objects.isNull;

public class MessageHelper {
    private MessageHelper() {
    }

    public static Object getField(MessageOrBuilder messageOrBuilder, String fieldName) {
        if (isNull(messageOrBuilder) || StringUtils.isBlank(fieldName)) {
            throw new ValidationFrostFSException(SOME_PARAM_IS_MISSING);
        }

        return messageOrBuilder.getField(messageOrBuilder.getDescriptorForType().findFieldByName(fieldName));
    }

    public static void setField(Message.Builder builder, String fieldName, Object value) {
        if (isNull(builder) || StringUtils.isBlank(fieldName) || isNull(value)) {
            throw new ValidationFrostFSException(SOME_PARAM_IS_MISSING);
        }

        builder.setField(builder.getDescriptorForType().findFieldByName(fieldName), value);
    }
}
