package info.frostfs.sdk.testgenerator;

import com.google.protobuf.ByteString;
import frostfs.ape.Types;
import frostfs.apemanager.Service;
import info.frostfs.sdk.Helper;
import org.bouncycastle.util.encoders.Base64;

import java.util.ArrayList;

public class ApeManagerGenerator {
    private static final String BASE64_CHAIN = "AAAaY2hhaW4taWQtdGVzdAIAAAICKgACHm5hdGl2ZTpvYmplY3QvKgAAAA==";

    private static ByteString generateChainID() {
        return ByteString.copyFrom(Helper.getByteArrayFromHex("616c6c6f774f626a476574436e72"));
    }

    private static Types.Chain generateRawChain() {
        return Types.Chain.newBuilder()
                .setRaw(ByteString.copyFrom(Base64.decode(BASE64_CHAIN)))
                .build();
    }

    private static Iterable<Types.Chain> generateRawChains(int size) {
        var list = new ArrayList<Types.Chain>(size);
        for (int i = 0; i < size; i++) {
            list.add(generateRawChain());
        }

        return list;
    }

    private static Types.ChainTarget generateChainTarget() {
        return Types.ChainTarget.newBuilder()
                .setType(Types.TargetType.CONTAINER)
                .setName("BzQw5HH3feoxFDD5tCT87Y1726qzgLfxEE7wgtoRzB3R")
                .build();
    }

    public static Service.AddChainRequest.Body generateAddChainRequestBody() {
        return Service.AddChainRequest.Body.newBuilder()
                .setTarget(generateChainTarget())
                .setChain(generateRawChain())
                .build();
    }

    public static Service.AddChainRequest generateAddChainRequest() {
        return Service.AddChainRequest.newBuilder()
                .setBody(generateAddChainRequestBody())
                .setMetaHeader(SessionGenerator.generateRequestMetaHeader())
                .setVerifyHeader(SessionGenerator.generateRequestVerificationHeader())
                .build();
    }

    public static Service.AddChainResponse.Body generateAddChainResponseBody() {
        return Service.AddChainResponse.Body.newBuilder()
                .setChainId(generateChainID())
                .build();
    }

    public static Service.AddChainResponse generateAddChainResponse() {
        return Service.AddChainResponse.newBuilder()
                .setBody(generateAddChainResponseBody())
                .setMetaHeader(SessionGenerator.generateResponseMetaHeader())
                .setVerifyHeader(SessionGenerator.generateResponseVerificationHeader())
                .build();
    }

    public static Service.RemoveChainRequest.Body generateRemoveChainRequestBody() {
        return Service.RemoveChainRequest.Body.newBuilder()
                .setChainId(generateChainID())
                .setTarget(generateChainTarget())
                .build();
    }

    public static Service.RemoveChainRequest generateRemoveChainRequest() {
        return Service.RemoveChainRequest.newBuilder()
                .setBody(generateRemoveChainRequestBody())
                .setMetaHeader(SessionGenerator.generateRequestMetaHeader())
                .setVerifyHeader(SessionGenerator.generateRequestVerificationHeader())
                .build();
    }

    public static Service.RemoveChainResponse.Body generateRemoveChainResponseBody() {
        return Service.RemoveChainResponse.Body.getDefaultInstance();
    }

    public static Service.RemoveChainResponse generateRemoveChainResponse() {
        return Service.RemoveChainResponse.newBuilder()
                .setBody(generateRemoveChainResponseBody())
                .setMetaHeader(SessionGenerator.generateResponseMetaHeader())
                .setVerifyHeader(SessionGenerator.generateResponseVerificationHeader())
                .build();
    }

    public static Service.ListChainsRequest.Body generateListChainsRequestBody() {
        return Service.ListChainsRequest.Body.newBuilder()
                .setTarget(generateChainTarget())
                .build();
    }

    public static Service.ListChainsRequest generateListChainsRequest() {
        return Service.ListChainsRequest.newBuilder()
                .setBody(generateListChainsRequestBody())
                .setMetaHeader(SessionGenerator.generateRequestMetaHeader())
                .setVerifyHeader(SessionGenerator.generateRequestVerificationHeader())
                .build();
    }

    public static Service.ListChainsResponse.Body generateListChainsResponseBody() {
        return Service.ListChainsResponse.Body.newBuilder()
                .addAllChains(generateRawChains(10))
                .build();
    }

    public static Service.ListChainsResponse generateListChainsResponse() {
        return Service.ListChainsResponse.newBuilder()
                .setBody(generateListChainsResponseBody())
                .setMetaHeader(SessionGenerator.generateResponseMetaHeader())
                .setVerifyHeader(SessionGenerator.generateResponseVerificationHeader())
                .build();
    }
}
