package info.frostfs.sdk;

import info.frostfs.sdk.exceptions.ValidationFrostFSException;
import org.junit.jupiter.api.Test;

import static org.assertj.core.api.Assertions.assertThat;
import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertThrows;

public class KeyExtensionTest {
    private static final byte[] PRIVATE_KEY = new byte[]{
            -128, -5, 30, -36, -118, 85, -67, -6, 81, 43, 93, -38, 106, 21, -88, 127, 15, 125, -79, -17, -40, 77, -15,
            122, -88, 72, 109, -47, 125, -80, -40, -38
    };
    private static final byte[] PUBLIC_KEY = new byte[]{
            3, 80, -111, 65, 23, 36, -4, -69, 80, 102, 98, -13, 13, -79, 24, 126, -116, 90, 56, 44, 127, 106, -125, 99,
            -48, -94, 104, 81, 120, -110, 30, 80, 70
    };
    private static final byte[] UNCOMPRESSED_PUBLIC_KEY = new byte[]{
            4, 80, -111, 65, 23, 36, -4, -69, 80, 102, 98, -13, 13, -79, 24, 126, -116, 90, 56, 44, 127, 106, -125, 99,
            -48, -94, 104, 81, 120, -110, 30, 80, 70, 76, -18, 53, -9, 79, -8, -25, -69, 12, 89, -103, 15, 126, 118,
            -68, -73, 65, -57, -26, 75, 4, -51, -40, -20, 75, 89, -59, 111, 96, -80, 56, 13
    };

    @Test
    void loadPrivateKey_success() {
        //When
        var privateKey = KeyExtension.loadPrivateKey(PRIVATE_KEY);

        //Then
        assertThat(privateKey.getEncoded()).hasSize(67).endsWith(PRIVATE_KEY);
        assertEquals("EC", privateKey.getAlgorithm());
        assertEquals("PKCS#8", privateKey.getFormat());
    }

    @Test
    void loadPrivateKey_wrong() {
        //When + Then
        assertThrows(ValidationFrostFSException.class, () -> KeyExtension.loadPrivateKey(null));
        assertThrows(ValidationFrostFSException.class, () -> KeyExtension.loadPrivateKey(new byte[]{}));
    }

    @Test
    void getPublicKeyFromBytes_success() {
        //When
        var publicKey = KeyExtension.getPublicKeyFromBytes(PUBLIC_KEY);

        //Then
        assertThat(publicKey.getEncoded()).hasSize(91).endsWith(UNCOMPRESSED_PUBLIC_KEY);
        assertEquals("EC", publicKey.getAlgorithm());
        assertEquals("X.509", publicKey.getFormat());
    }

    @Test
    void getPublicKeyFromBytes_wrong() {
        //When + Then
        assertThrows(ValidationFrostFSException.class, () -> KeyExtension.getPublicKeyFromBytes(null));
        assertThrows(ValidationFrostFSException.class, () -> KeyExtension.getPublicKeyFromBytes(new byte[]{}));
        assertThrows(ValidationFrostFSException.class, () -> KeyExtension.getPublicKeyFromBytes(PRIVATE_KEY));
    }
}
