package info.frostfs.sdk.dto.object;

import info.frostfs.sdk.dto.container.ContainerId;
import info.frostfs.sdk.enums.ObjectType;
import info.frostfs.sdk.exceptions.ValidationFrostFSException;
import lombok.Getter;
import lombok.Setter;

import java.util.List;

import static info.frostfs.sdk.constants.ErrorConst.INPUT_PARAM_IS_MISSING_TEMPLATE;
import static java.util.Objects.isNull;

@Getter
@Setter
public class ObjectFrostFS {
    private final ObjectHeader header;
    private ObjectId objectId;
    private byte[] payload;
    private ObjectReader objectReader;

    public ObjectFrostFS(ObjectHeader header, ObjectId objectId, byte[] payload) {
        if (isNull(header)) {
            throw new ValidationFrostFSException(
                    String.format(INPUT_PARAM_IS_MISSING_TEMPLATE, ObjectHeader.class.getName())
            );
        }

        this.header = header;
        this.objectId = objectId;
        this.payload = payload;
    }

    public ObjectFrostFS(ContainerId containerId, byte[] payload) {
        this.payload = payload;
        this.header = new ObjectHeader(containerId);
    }

    public ObjectFrostFS(ContainerId containerId, byte[] payload, ObjectType objectType) {
        this.payload = payload;
        this.header = new ObjectHeader(containerId, objectType);
    }

    public void setSplit(Split split) {
        header.setSplit(split);
    }

    public void addAttribute(String key, String value) {
        header.getAttributes().add(new ObjectAttribute(key, value));
    }

    public void addAttribute(ObjectAttribute attribute) {
        header.getAttributes().add(attribute);
    }

    public void addAttributes(List<ObjectAttribute> attributes) {
        header.getAttributes().addAll(attributes);
    }

    public void setParent(LargeObject largeObject) {
        if (isNull(header.getSplit())) {
            throw new ValidationFrostFSException(
                    String.format(INPUT_PARAM_IS_MISSING_TEMPLATE, Split.class.getName())
            );
        }

        header.getSplit().setParentHeader(largeObject.getHeader());
    }
}
