package info.frostfs.sdk.dto.object;

import info.frostfs.sdk.dto.container.ContainerId;
import info.frostfs.sdk.dto.netmap.Version;
import info.frostfs.sdk.enums.ObjectType;
import info.frostfs.sdk.exceptions.ValidationFrostFSException;
import lombok.Getter;
import lombok.Setter;

import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

import static info.frostfs.sdk.constants.ErrorConst.FIELDS_DELIMITER_OR;
import static info.frostfs.sdk.constants.ErrorConst.INPUT_PARAM_IS_MISSING_TEMPLATE;
import static java.util.Objects.isNull;

@Getter
@Setter
public class ObjectHeader {
    private final ContainerId containerId;
    private final ObjectType objectType;
    private List<ObjectAttribute> attributes;
    private Version version;
    private OwnerId ownerId;
    private long payloadLength;
    private byte[] payloadCheckSum;
    private Split split;

    public ObjectHeader(ContainerId containerId, ObjectType objectType,
                        List<ObjectAttribute> attributes, long payloadLength, Version version) {
        if (isNull(containerId) || isNull(objectType)) {
            throw new ValidationFrostFSException(
                    String.format(
                            INPUT_PARAM_IS_MISSING_TEMPLATE,
                            String.join(FIELDS_DELIMITER_OR, ContainerId.class.getName(), ObjectType.class.getName())
                    )
            );
        }

        this.attributes = attributes;
        this.containerId = containerId;
        this.payloadLength = payloadLength;
        this.objectType = objectType;
        this.version = version;
    }

    public ObjectHeader(ContainerId containerId, ObjectType objectType, ObjectAttribute... attributes) {
        this(
                containerId,
                objectType,
                Arrays.stream(attributes).collect(Collectors.toList()),
                0L,
                null
        );
    }

    public ObjectHeader(ContainerId containerId, ObjectAttribute... attributes) {
        this(containerId, ObjectType.REGULAR, attributes);
    }

    public void increasePayloadLength(long payloadLength) {
        this.payloadLength += payloadLength;
    }

    public void setSplit(Split split) {
        if (isNull(split)) {
            throw new ValidationFrostFSException(
                    String.format(INPUT_PARAM_IS_MISSING_TEMPLATE, Split.class.getName())
            );
        }

        this.split = split;
    }
}
