package info.frostfs.sdk.enums;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

public enum ConditionType {
    COND_STRING_EQUALS(0),
    COND_STRING_NOT_EQUALS(1),
    COND_STRING_EQUALS_IGNORE_CASE(2),
    COND_STRING_NOT_EQUALS_IGNORE_CASE(3),

    COND_STRING_LIKE(4),
    COND_STRING_NOT_LIKE(5),

    COND_STRING_LESS_THAN(6),
    COND_STRING_LESS_THAN_EQUALS(7),
    COND_STRING_GREATER_THAN(8),
    COND_STRING_GREATER_THAN_EQUALS(9),

    COND_NUMERIC_EQUALS(10),
    COND_NUMERIC_NOT_EQUALS(11),

    COND_NUMERIC_LESS_THAN(12),
    COND_NUMERIC_LESS_THAN_EQUALS(13),
    COND_NUMERIC_GREATER_THAN(14),
    COND_NUMERIC_GREATER_THAN_EQUALS(15),

    COND_SLICE_CONTAINS(16),

    COND_IP_ADDRESS(17),
    COND_NOT_IP_ADDRESS(18),
    ;

    private static final Map<Integer, ConditionType> ENUM_MAP_BY_VALUE;

    static {
        Map<Integer, ConditionType> map = new HashMap<>();
        for (ConditionType conditionType : ConditionType.values()) {
            map.put(conditionType.value, conditionType);
        }
        ENUM_MAP_BY_VALUE = Collections.unmodifiableMap(map);
    }

    public final int value;

    ConditionType(int value) {
        this.value = value;
    }

    public static ConditionType get(int value) {
        return ENUM_MAP_BY_VALUE.get(value);
    }
}
