package info.frostfs.sdk.enums;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

public enum StatusCode {
    SUCCESS(0),
    INTERNAL(1024),
    WRONG_MAGIC_NUMBER(1025),
    SIGNATURE_VERIFICATION_FAILURE(1026),
    NODE_UNDER_MAINTENANCE(1027),
    OBJECT_ACCESS_DENIED(2048),
    OBJECT_NOT_FOUND(2049),
    OBJECT_LOCKED(2050),
    LOCK_NOT_REGULAR_OBJECT(2051),
    OBJECT_ALREADY_REMOVED(2052),
    OUT_OF_RANGE(2053),
    CONTAINER_NOT_FOUND(3072),
    E_ACL_NOT_FOUND(3073),
    CONTAINER_ACCESS_DENIED(3074),
    TOKEN_NOT_FOUND(4096),
    TOKEN_EXPIRED(4097),
    APE_MANAGER_ACCESS_DENIED(5120),
    ;

    private static final Map<Integer, StatusCode> ENUM_MAP_BY_VALUE;

    static {
        Map<Integer, StatusCode> map = new HashMap<>();
        for (StatusCode statusCode : StatusCode.values()) {
            map.put(statusCode.value, statusCode);
        }
        ENUM_MAP_BY_VALUE = Collections.unmodifiableMap(map);
    }

    public final int value;

    StatusCode(int value) {
        this.value = value;
    }

    public static StatusCode get(int value) {
        return ENUM_MAP_BY_VALUE.get(value);
    }
}
