package info.frostfs.sdk.mappers.chain;

import frostfs.ape.Types;
import info.frostfs.sdk.dto.chain.ChainTarget;
import info.frostfs.sdk.exceptions.ProcessFrostFSException;

import static info.frostfs.sdk.constants.ErrorConst.UNKNOWN_ENUM_VALUE_TEMPLATE;
import static java.util.Objects.isNull;

public class ChainTargetMapper {
    private ChainTargetMapper() {
    }

    public static Types.ChainTarget toGrpcMessage(ChainTarget chainTarget) {
        if (isNull(chainTarget)) {
            return null;
        }

        var targetType = Types.TargetType.forNumber(chainTarget.getType().value);
        if (isNull(targetType)) {
            throw new ProcessFrostFSException(String.format(
                    UNKNOWN_ENUM_VALUE_TEMPLATE,
                    Types.ChainTarget.class.getName(),
                    chainTarget.getType().name()
            ));
        }

        return Types.ChainTarget.newBuilder()
                .setType(targetType)
                .setName(chainTarget.getName())
                .build();
    }
}
