package info.frostfs.sdk.mappers.container;

import com.google.protobuf.ByteString;
import frostfs.container.Types;
import info.frostfs.sdk.dto.container.Container;
import info.frostfs.sdk.mappers.netmap.PlacementPolicyMapper;
import info.frostfs.sdk.mappers.netmap.VersionMapper;
import info.frostfs.sdk.mappers.object.OwnerIdMapper;

import java.util.Optional;
import java.util.stream.Collectors;

import static info.frostfs.sdk.UuidExtension.asBytes;
import static info.frostfs.sdk.UuidExtension.asUuid;
import static java.util.Objects.isNull;

public class ContainerMapper {
    private ContainerMapper() {
    }

    public static Types.Container.Builder toGrpcMessage(Container container) {
        if (isNull(container)) {
            return null;
        }

        var attributes = container.getAttributes().entrySet().stream()
                .map(entry ->
                        Types.Container.Attribute.newBuilder()
                                .setKey(entry.getKey())
                                .setValue(entry.getValue())
                                .build()
                )
                .collect(Collectors.toList());

        var containerGrpc = Types.Container.newBuilder()
                .setPlacementPolicy(PlacementPolicyMapper.toGrpcMessage(container.getPlacementPolicy()))
                .setNonce(ByteString.copyFrom(asBytes(container.getNonce())))
                .addAllAttributes(attributes);

        Optional.ofNullable(OwnerIdMapper.toGrpcMessage(container.getOwnerId())).ifPresent(containerGrpc::setOwnerId);
        Optional.ofNullable(VersionMapper.toGrpcMessage(container.getVersion())).ifPresent(containerGrpc::setVersion);

        return containerGrpc;
    }

    public static Container toModel(Types.Container containerGrpc) {
        if (isNull(containerGrpc) || containerGrpc.getSerializedSize() == 0) {
            return null;
        }

        var attributes = containerGrpc.getAttributesList().stream()
                .collect(Collectors.toMap(Types.Container.Attribute::getKey, Types.Container.Attribute::getValue));

        return new Container(
                asUuid(containerGrpc.getNonce().toByteArray()),
                PlacementPolicyMapper.toModel(containerGrpc.getPlacementPolicy()),
                VersionMapper.toModel(containerGrpc.getVersion()),
                OwnerIdMapper.toModel(containerGrpc.getOwnerId()),
                attributes
        );
    }
}
