package info.frostfs.sdk.mappers.netmap;

import frostfs.netmap.Types;
import info.frostfs.sdk.dto.netmap.PlacementPolicy;

import static java.util.Objects.isNull;

public class PlacementPolicyMapper {
    private PlacementPolicyMapper() {
    }

    public static Types.PlacementPolicy toGrpcMessage(PlacementPolicy placementPolicy) {
        if (isNull(placementPolicy)) {
            return null;
        }

        return Types.PlacementPolicy.newBuilder()
                .setUnique(placementPolicy.isUnique())
                .setContainerBackupFactor(placementPolicy.getBackupFactory())
                .addAllFilters(FilterMapper.toGrpcMessages(placementPolicy.getFilters()))
                .addAllSelectors(SelectorMapper.toGrpcMessages(placementPolicy.getSelectors()))
                .addAllReplicas(ReplicaMapper.toGrpcMessages(placementPolicy.getReplicas()))
                .build();
    }

    public static PlacementPolicy toModel(Types.PlacementPolicy placementPolicy) {
        if (isNull(placementPolicy) || placementPolicy.getSerializedSize() == 0) {
            return null;
        }

        return new PlacementPolicy(
                ReplicaMapper.toModels(placementPolicy.getReplicasList()),
                placementPolicy.getUnique(),
                placementPolicy.getContainerBackupFactor(),
                FilterMapper.toModels(placementPolicy.getFiltersList()),
                SelectorMapper.toModels(placementPolicy.getSelectorsList())
        );
    }
}
