package info.frostfs.sdk.mappers.netmap;

import frostfs.netmap.Types;
import info.frostfs.sdk.dto.netmap.Replica;
import info.frostfs.sdk.exceptions.ValidationFrostFSException;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.ArrayUtils;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

import static info.frostfs.sdk.constants.ErrorConst.INPUT_PARAM_IS_MISSING_TEMPLATE;
import static java.util.Objects.isNull;

public class ReplicaMapper {
    private ReplicaMapper() {
    }

    public static List<Types.Replica> toGrpcMessages(Replica[] replicas) {
        if (ArrayUtils.isEmpty(replicas)) {
            return Collections.emptyList();
        }

        return Arrays.stream(replicas).map(ReplicaMapper::toGrpcMessage).collect(Collectors.toList());
    }

    public static Types.Replica toGrpcMessage(Replica replica) {
        if (isNull(replica)) {
            throw new ValidationFrostFSException(
                    String.format(INPUT_PARAM_IS_MISSING_TEMPLATE, Replica.class.getName())
            );
        }

        return Types.Replica.newBuilder()
                .setCount(replica.getCount())
                .setSelector(replica.getSelector())
                .build();
    }

    public static Replica[] toModels(List<Types.Replica> filters) {
        if (CollectionUtils.isEmpty(filters)) {
            return null;
        }

        return filters.stream().map(ReplicaMapper::toModel).toArray(Replica[]::new);
    }

    public static Replica toModel(Types.Replica replica) {
        if (isNull(replica) || replica.getSerializedSize() == 0) {
            return null;
        }

        return new Replica(replica.getCount(), replica.getSelector());
    }
}
