package info.frostfs.sdk.mappers.object;

import frostfs.object.Types;
import info.frostfs.sdk.dto.object.ObjectAttribute;
import org.apache.commons.collections4.CollectionUtils;

import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

import static java.util.Objects.isNull;

public class ObjectAttributeMapper {
    private ObjectAttributeMapper() {
    }

    public static List<Types.Header.Attribute> toGrpcMessages(List<ObjectAttribute> attributes) {
        if (CollectionUtils.isEmpty(attributes)) {
            return Collections.emptyList();
        }

        return attributes.stream().map(ObjectAttributeMapper::toGrpcMessage).collect(Collectors.toList());
    }

    public static Types.Header.Attribute toGrpcMessage(ObjectAttribute attribute) {
        if (isNull(attribute)) {
            return null;
        }

        return Types.Header.Attribute.newBuilder()
                .setKey(attribute.getKey())
                .setValue(attribute.getValue())
                .build();
    }

    public static ObjectAttribute toModel(Types.Header.Attribute attribute) {
        if (isNull(attribute) || attribute.getSerializedSize() == 0) {
            return null;
        }

        return new ObjectAttribute(attribute.getKey(), attribute.getValue());
    }
}
