package info.frostfs.sdk.mappers.object;

import frostfs.object.Types;
import info.frostfs.sdk.dto.object.ObjectId;
import info.frostfs.sdk.dto.object.SplitId;
import info.frostfs.sdk.dto.object.SplitInfo;

import static info.frostfs.sdk.UuidExtension.asUuid;
import static java.util.Objects.isNull;

public class SplitInfoMapper {
    private SplitInfoMapper() {
    }

    public static SplitInfo toModel(Types.SplitInfo splitInfo) {
        if (isNull(splitInfo) || splitInfo.getSerializedSize() == 0) {
            return null;
        }

        var splitId = new SplitId(asUuid(splitInfo.getSplitId().toByteArray()));
        var link = splitInfo.getLink().getSerializedSize() == 0
                ? null
                : new ObjectId(splitInfo.getLink().getValue().toByteArray());
        var lastPart = splitInfo.getLastPart().getSerializedSize() == 0
                ? null
                : new ObjectId(splitInfo.getLastPart().getValue().toByteArray());

        return new SplitInfo(splitInfo, splitId, link, lastPart);
    }
}
