package info.frostfs.sdk.mappers.response;

import frostfs.session.Types;
import info.frostfs.sdk.dto.response.MetaHeader;
import info.frostfs.sdk.exceptions.ValidationFrostFSException;
import info.frostfs.sdk.mappers.netmap.VersionMapper;

import static info.frostfs.sdk.constants.ErrorConst.INPUT_PARAM_IS_MISSING_TEMPLATE;
import static java.util.Objects.isNull;

public class MetaHeaderMapper {
    private MetaHeaderMapper() {
    }

    public static Types.RequestMetaHeader toGrpcMessage(MetaHeader metaHeader) {
        if (isNull(metaHeader)) {
            throw new ValidationFrostFSException(
                    String.format(INPUT_PARAM_IS_MISSING_TEMPLATE, MetaHeader.class.getName())
            );
        }

        return toGrpcMessageBuilder(metaHeader).build();
    }

    public static Types.RequestMetaHeader.Builder toGrpcMessageBuilder(MetaHeader metaHeader) {
        if (isNull(metaHeader)) {
            throw new ValidationFrostFSException(
                    String.format(INPUT_PARAM_IS_MISSING_TEMPLATE, MetaHeader.class.getName())
            );
        }

        return Types.RequestMetaHeader.newBuilder()
                .setVersion(VersionMapper.toGrpcMessage(metaHeader.getVersion()))
                .setEpoch(metaHeader.getEpoch())
                .setTtl(metaHeader.getTtl());
    }
}
