package info.frostfs.sdk.mappers.response;

import com.google.protobuf.ByteString;
import frostfs.refs.Types;
import info.frostfs.sdk.dto.response.Signature;
import info.frostfs.sdk.exceptions.ProcessFrostFSException;

import static info.frostfs.sdk.constants.ErrorConst.UNKNOWN_ENUM_VALUE_TEMPLATE;
import static java.util.Objects.isNull;

public class SignatureMapper {
    private SignatureMapper() {
    }

    public static Types.Signature toGrpcMessage(Signature signature) {
        if (isNull(signature)) {
            return null;
        }

        var scheme = Types.SignatureScheme.forNumber(signature.getScheme().value);
        if (isNull(scheme)) {
            throw new ProcessFrostFSException(String.format(
                    UNKNOWN_ENUM_VALUE_TEMPLATE,
                    Types.SignatureScheme.class.getName(),
                    signature.getScheme().name()
            ));
        }

        return Types.Signature.newBuilder()
                .setKey(ByteString.copyFrom(signature.getKey()))
                .setSign(ByteString.copyFrom(signature.getSign()))
                .setScheme(scheme)
                .build();
    }
}
