package info.frostfs.sdk.mappers.session;

import com.google.protobuf.CodedOutputStream;
import com.google.protobuf.InvalidProtocolBufferException;
import frostfs.session.Types;
import info.frostfs.sdk.exceptions.ProcessFrostFSException;
import info.frostfs.sdk.exceptions.ValidationFrostFSException;

import java.io.IOException;

import static info.frostfs.sdk.constants.ErrorConst.INPUT_PARAM_IS_MISSING_TEMPLATE;
import static java.util.Objects.isNull;

public class SessionMapper {
    private SessionMapper() {
    }

    public static byte[] serialize(Types.SessionToken token) {
        if (isNull(token)) {
            throw new ValidationFrostFSException(
                    String.format(INPUT_PARAM_IS_MISSING_TEMPLATE, Types.SessionToken.class.getName())
            );
        }

        try {
            byte[] bytes = new byte[token.getSerializedSize()];
            CodedOutputStream stream = CodedOutputStream.newInstance(bytes);
            token.writeTo(stream);
            return bytes;
        } catch (IOException exp) {
            throw new ProcessFrostFSException(exp.getMessage());
        }
    }

    public static Types.SessionToken deserializeSessionToken(byte[] bytes) {
        if (isNull(bytes) || bytes.length == 0) {
            throw new ValidationFrostFSException(
                    String.format(INPUT_PARAM_IS_MISSING_TEMPLATE, Types.SessionToken.class.getName())
            );
        }

        try {
            return Types.SessionToken.newBuilder().mergeFrom(bytes).build();
        } catch (InvalidProtocolBufferException exp) {
            throw new ProcessFrostFSException(exp.getMessage());
        }
    }
}
