package info.frostfs.sdk.mappers.chain;

import frostfs.ape.Types;
import info.frostfs.sdk.dto.chain.ChainTarget;
import info.frostfs.sdk.enums.TargetType;
import info.frostfs.sdk.exceptions.ProcessFrostFSException;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.EnumSource;
import org.mockito.MockedStatic;

import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.Mockito.mockStatic;

public class ChainTargetMapperTest {

    @ParameterizedTest
    @EnumSource(value = TargetType.class)
    void toGrpcMessage_success(TargetType targetType) {
        //Given
        var target = new ChainTarget("BzQw5HH3feoxFDD5tCT87Y1726qzgLfxEE7wgtoRzB3R", targetType);


        //When
        var result = ChainTargetMapper.toGrpcMessage(target);

        //Then
        assertNotNull(result);
        assertEquals(target.getName(), result.getName());
        assertEquals(target.getType().value, result.getTypeValue());
    }

    @Test
    void toGrpcMessage_null() {
        //When + Then
        assertNull(ChainTargetMapper.toGrpcMessage(null));
    }

    @Test
    void toGrpcMessage_notValidScheme() {
        //Given
        var target = new ChainTarget("BzQw5HH3feoxFDD5tCT87Y1726qzgLfxEE7wgtoRzB3R", TargetType.UNDEFINED);

        //When + Then
        try (MockedStatic<Types.TargetType> mockStatic = mockStatic(Types.TargetType.class)) {
            mockStatic.when(() -> Types.TargetType.forNumber(target.getType().value))
                    .thenReturn(null);

            assertThrows(ProcessFrostFSException.class, () -> ChainTargetMapper.toGrpcMessage(target));
        }
    }
}
