package info.frostfs.sdk.mappers.netmap;

import frostfs.netmap.Types;
import info.frostfs.sdk.dto.netmap.PlacementPolicy;
import info.frostfs.sdk.dto.netmap.Replica;
import org.junit.jupiter.api.Test;

import static org.junit.jupiter.api.Assertions.*;

public class PlacementPolicyMapperTest {

    @Test
    void toGrpcMessage_success() {
        //Given
        var replica1 = new Replica(1, "test1");
        var replica2 = new Replica(2, "test2");

        var placementPolicy = new PlacementPolicy(new Replica[]{replica1, replica2}, true, 1);

        //When
        var result = PlacementPolicyMapper.toGrpcMessage(placementPolicy);

        //Then
        assertNotNull(result);
        assertEquals(placementPolicy.isUnique(), result.getUnique());
        assertEquals(placementPolicy.getBackupFactory(), result.getContainerBackupFactor());
        assertEquals(placementPolicy.getReplicas().length, result.getReplicasCount());
        assertEquals(replica1.getCount(), result.getReplicas(0).getCount());
        assertEquals(replica1.getSelector(), result.getReplicas(0).getSelector());
        assertEquals(replica2.getCount(), result.getReplicas(1).getCount());
        assertEquals(replica2.getSelector(), result.getReplicas(1).getSelector());
    }

    @Test
    void toGrpcMessage_null() {
        //When + Then
        assertNull(PlacementPolicyMapper.toGrpcMessage(null));
    }

    @Test
    void toModel_success() {
        //Given
        var replica1 = Types.Replica.newBuilder()
                .setCount(1)
                .setSelector("test1")
                .build();

        var replica2 = Types.Replica.newBuilder()
                .setCount(2)
                .setSelector("test2")
                .build();

        var placementPolicy = Types.PlacementPolicy.newBuilder()
                .setUnique(true)
                .addReplicas(replica1)
                .addReplicas(replica2)
                .setContainerBackupFactor(1)
                .build();

        //When
        var result = PlacementPolicyMapper.toModel(placementPolicy);

        //Then
        assertNotNull(result);
        assertEquals(placementPolicy.getUnique(), result.isUnique());
        assertEquals(placementPolicy.getContainerBackupFactor(), result.getBackupFactory());
        assertEquals(placementPolicy.getReplicasCount(), result.getReplicas().length);
        assertEquals(replica1.getCount(), result.getReplicas()[0].getCount());
        assertEquals(replica1.getSelector(), result.getReplicas()[0].getSelector());
        assertEquals(replica2.getCount(), result.getReplicas()[1].getCount());
        assertEquals(replica2.getSelector(), result.getReplicas()[1].getSelector());
    }

    @Test
    void toModel_null() {
        //When + Then
        assertNull(PlacementPolicyMapper.toModel(null));
        assertNull(PlacementPolicyMapper.toModel(Types.PlacementPolicy.getDefaultInstance()));
    }
}
