package info.frostfs.sdk.mappers.netmap;

import frostfs.netmap.Types;
import info.frostfs.sdk.dto.netmap.Replica;
import info.frostfs.sdk.exceptions.ValidationFrostFSException;
import org.junit.jupiter.api.Test;

import static org.junit.jupiter.api.Assertions.*;

public class ReplicaMapperTest {

    @Test
    void toGrpcMessage_success() {
        //Given
        var replica = new Replica(1);

        //When
        var result = ReplicaMapper.toGrpcMessage(replica);

        //Then
        assertNotNull(result);
        assertEquals(replica.getCount(), result.getCount());
        assertEquals(replica.getSelector(), result.getSelector());
    }

    @Test
    void toGrpcMessage_null() {
        //When + Then
        assertThrows(ValidationFrostFSException.class, () -> ReplicaMapper.toGrpcMessage(null));
    }

    @Test
    void toModel_success() {
        //Given
        var replica = Types.Replica.newBuilder()
                .setCount(1)
                .setSelector("")
                .build();

        //When
        var result = ReplicaMapper.toModel(replica);

        //Then
        assertNotNull(result);
        assertEquals(replica.getCount(), result.getCount());
        assertEquals(replica.getSelector(), result.getSelector());
    }

    @Test
    void toModel_null() {
        //When + Then
        assertNull(ReplicaMapper.toModel(null));
        assertNull(ReplicaMapper.toModel(Types.Replica.getDefaultInstance()));
    }
}
