package info.frostfs.sdk.mappers.netmap;

import frostfs.refs.Types;
import info.frostfs.sdk.dto.netmap.Version;
import org.junit.jupiter.api.Test;

import static org.junit.jupiter.api.Assertions.*;

public class VersionMapperTest {

    @Test
    void toGrpcMessage_success() {
        //Given
        var version = new Version(1, 2);

        //When
        var result = VersionMapper.toGrpcMessage(version);

        //Then
        assertNotNull(result);
        assertEquals(version.getMajor(), result.getMajor());
        assertEquals(version.getMinor(), result.getMinor());
    }

    @Test
    void toGrpcMessage_null() {
        //When + Then
        assertNull(VersionMapper.toGrpcMessage(null));
    }

    @Test
    void toModel_success() {
        //Given
        var version = Types.Version.newBuilder()
                .setMajor(1)
                .setMinor(2)
                .build();

        //When
        var result = VersionMapper.toModel(version);

        //Then
        assertNotNull(result);
        assertEquals(version.getMajor(), result.getMajor());
        assertEquals(version.getMinor(), result.getMinor());
    }

    @Test
    void toModel_null() {
        //When + Then
        assertNull(VersionMapper.toModel(null));
        assertNull(VersionMapper.toModel(Types.Version.getDefaultInstance()));
    }
}
