package info.frostfs.sdk.mappers.object;

import com.google.protobuf.ByteString;
import frostfs.object.Types;
import info.frostfs.sdk.dto.container.ContainerId;
import info.frostfs.sdk.dto.object.ObjectId;
import org.junit.jupiter.api.Test;

import static org.assertj.core.api.Assertions.assertThat;
import static org.junit.jupiter.api.Assertions.*;

public class ObjectFrostFSMapperTest {
    @Test
    void toModel_success() {
        //Given
        var containerId = new ContainerId("EQGx2QeYHJb53uRwYGzcQaW191sZpdNrjutk6veUSV2R");
        var cid = frostfs.refs.Types.ContainerID.newBuilder()
                .setValue(ByteString.copyFrom(containerId.toHash()))
                .build();
        var version = frostfs.refs.Types.Version.newBuilder()
                .setMajor(1)
                .setMinor(2)
                .build();
        var header = Types.Header.newBuilder()
                .setObjectType(Types.ObjectType.REGULAR)
                .setVersion(version)
                .setContainerId(cid)
                .setPayloadLength(10)
                .build();

        var objectId = new ObjectId("85orCLKSu3X1jGiTFmwmTUsBU88RBARNwuRwrEy5pyww");
        var object = Types.Object.newBuilder()
                .setObjectId(ObjectIdMapper.toGrpcMessage(objectId))
                .setPayload(ByteString.copyFrom(new byte[]{1, 2, 3, 4, 5}))
                .setHeader(header)
                .build();

        //When
        var result = ObjectFrostFSMapper.toModel(object);

        //Then
        assertNotNull(result);
        assertEquals(objectId.getValue(), result.getObjectId().getValue());
        assertThat(result.getPayload()).containsExactly(object.getPayload().toByteArray());
        assertThat(result.getHeader().getAttributes()).hasSize(0);
        assertEquals(header.getObjectTypeValue(), result.getHeader().getObjectType().value);
        assertEquals(version.getMajor(), result.getHeader().getVersion().getMajor());
        assertEquals(version.getMinor(), result.getHeader().getVersion().getMinor());
        assertEquals(header.getPayloadLength(), result.getHeader().getPayloadLength());
        assertEquals(containerId.getValue(), result.getHeader().getContainerId().getValue());
    }

    @Test
    void toModel_null() {
        //When + Then
        assertNull(ObjectFrostFSMapper.toModel(null));
        assertNull(ObjectFrostFSMapper.toModel(Types.Object.getDefaultInstance()));
    }

}
