package info.frostfs.sdk.mappers.object;

import com.google.protobuf.ByteString;
import frostfs.object.Types;
import info.frostfs.sdk.dto.object.ObjectId;
import info.frostfs.sdk.dto.object.SplitId;
import org.junit.jupiter.api.Test;

import static org.assertj.core.api.Assertions.assertThat;
import static org.junit.jupiter.api.Assertions.*;

public class SplitInfoMapperTest {
    @Test
    void toModel_successLastPart() {
        //Given
        var splitId = new SplitId();
        var objectId = new ObjectId("85orCLKSu3X1jGiTFmwmTUsBU88RBARNwuRwrEy5pyww");
        var splitInfo = Types.SplitInfo.newBuilder()
                .setSplitId(ByteString.copyFrom(splitId.toBinary()))
                .setLastPart(ObjectIdMapper.toGrpcMessage(objectId))
                .build();

        //When
        var result = SplitInfoMapper.toModel(splitInfo);

        //Then
        assertNotNull(result);
        assertNull(result.getLink());
        assertThat(result.getSplitInfo()).isEqualTo(splitInfo);
        assertThat(result.getSplitId().toBinary()).containsExactly(splitId.toBinary());
        assertEquals(objectId.getValue(), result.getLastPart().getValue());
    }

    @Test
    void toModel_successLink() {
        //Given
        var splitId = new SplitId();
        var objectId = new ObjectId("85orCLKSu3X1jGiTFmwmTUsBU88RBARNwuRwrEy5pyww");
        var splitInfo = Types.SplitInfo.newBuilder()
                .setSplitId(ByteString.copyFrom(splitId.toBinary()))
                .setLink(ObjectIdMapper.toGrpcMessage(objectId))
                .build();

        //When
        var result = SplitInfoMapper.toModel(splitInfo);

        //Then
        assertNotNull(result);
        assertNull(result.getLastPart());
        assertThat(result.getSplitInfo()).isEqualTo(splitInfo);
        assertThat(result.getSplitId().toBinary()).containsExactly(splitId.toBinary());
        assertEquals(objectId.getValue(), result.getLink().getValue());
    }

    @Test
    void toModel_null() {
        //When + Then
        assertNull(SplitInfoMapper.toModel(null));
        assertNull(SplitInfoMapper.toModel(Types.SplitInfo.getDefaultInstance()));
    }
}
