package info.frostfs.sdk.mappers.object.patch;

import info.frostfs.sdk.dto.object.patch.Range;
import org.apache.commons.collections4.CollectionUtils;
import org.junit.jupiter.api.Test;

import java.util.Arrays;
import java.util.Collections;

import static org.assertj.core.api.Assertions.assertThat;
import static org.junit.jupiter.api.Assertions.*;

public class RangeMapperTest {

    @Test
    void toGrpcMessages_success() {
        //Given
        var range1 = new Range(1, 10);
        var range2 = new Range(2, 20);

        //When
        var result = RangeMapper.toGrpcMessages(Arrays.asList(range1, range2));

        //Then
        assertNotNull(result);
        assertThat(result).isNotNull().hasSize(2);
        assertEquals(range1.getOffset(), result.get(0).getOffset());
        assertEquals(range1.getLength(), result.get(0).getLength());
        assertEquals(range2.getOffset(), result.get(1).getOffset());
        assertEquals(range2.getLength(), result.get(1).getLength());
    }

    @Test
    void toGrpcMessages_null() {
        //When + Then
        assertTrue(CollectionUtils.isEmpty(RangeMapper.toGrpcMessages(null)));
        assertTrue(CollectionUtils.isEmpty(RangeMapper.toGrpcMessages(Collections.emptyList())));
    }

    @Test
    void toGrpcMessage_success() {
        //Given
        var range = new Range(1, 10);

        //When
        var result = RangeMapper.toGrpcMessage(range);

        //Then
        assertNotNull(result);
        assertEquals(range.getOffset(), result.getOffset());
        assertEquals(range.getLength(), result.getLength());
    }

    @Test
    void toGrpcMessage_null() {
        //When + Then
        assertNull(RangeMapper.toGrpcMessage(null));
    }
}
