package info.frostfs.sdk.mappers.response;

import frostfs.session.Types;
import info.frostfs.sdk.dto.netmap.Version;
import info.frostfs.sdk.dto.response.MetaHeader;
import info.frostfs.sdk.exceptions.ValidationFrostFSException;
import org.junit.jupiter.api.Test;

import static org.junit.jupiter.api.Assertions.*;

public class MetaHeaderMapperTest {

    @Test
    void toGrpcMessage_success() {
        //Given
        var version = new Version(1, 2);
        var metaHeader = new MetaHeader(version, 1, 2);

        //When
        Types.RequestMetaHeader result = MetaHeaderMapper.toGrpcMessage(metaHeader);

        //Then
        assertNotNull(result);
        assertEquals(metaHeader.getEpoch(), result.getEpoch());
        assertEquals(metaHeader.getTtl(), result.getTtl());
        assertEquals(metaHeader.getVersion().getMajor(), result.getVersion().getMajor());
        assertEquals(metaHeader.getVersion().getMinor(), result.getVersion().getMinor());
    }

    @Test
    void toGrpcMessage_null() {
        //When + Then
        assertThrows(ValidationFrostFSException.class, () -> MetaHeaderMapper.toGrpcMessage(null));
    }

    @Test
    void toGrpcMessageBuilder_success() {
        //Given
        var version = new Version(1, 2);
        var metaHeader = new MetaHeader(version, 1, 2);

        //When
        Types.RequestMetaHeader.Builder result = MetaHeaderMapper.toGrpcMessageBuilder(metaHeader);

        //Then
        assertNotNull(result);
        assertEquals(metaHeader.getEpoch(), result.getEpoch());
        assertEquals(metaHeader.getTtl(), result.getTtl());
        assertEquals(metaHeader.getVersion().getMajor(), result.getVersion().getMajor());
        assertEquals(metaHeader.getVersion().getMinor(), result.getVersion().getMinor());
    }

    @Test
    void toGrpcMessageBuilder_null() {
        //When + Then
        assertThrows(ValidationFrostFSException.class, () -> MetaHeaderMapper.toGrpcMessageBuilder(null));
    }
}
